@@ -233,11 +233,13 @@ void __WFNamedTimerTask::dispatch()
 
 	mutex_.unlock();
 	if (ret < 0)
-		this->handle(SS_STATE_ERROR, errno);
+		this->handle(WFT_STATE_SYS_ERROR, errno);
 }
 
 void __WFNamedTimerTask::handle(int state, int error)
 {
+	bool canceled = true;
+
 	if (node_.task)
 	{
 		bool erased = false;
@@ -245,6 +247,7 @@ void __WFNamedTimerTask::handle(int state, int error)
 		__timer_map.mutex_.lock();
 		if (node_.task)
 		{
+			canceled = false;
 			erased = timers_->del(&node_, &__timer_map.root_);
 			node_.task = NULL;
 		}
@@ -254,6 +257,12 @@ void __WFNamedTimerTask::handle(int state, int error)
 			delete timers_;
 	}
 
+	if (canceled)
+	{
+		state = WFT_STATE_SYS_ERROR;
+		error = ECANCELED;
+	}
+
 	mutex_.lock();
 	mutex_.unlock();
 	this->__WFTimerTask::handle(state, error);
@@ -528,7 +537,7 @@ public:
 						  mailbox_callback_t&& cb);
 	WFMailboxTask *create(const std::string& name, mailbox_callback_t&& cb);
 
-	int send(const std::string& name, void *msg, size_t max);
+	int send(const std::string& name, void *const msg[], size_t max, int inc);
 	void send(MailboxList *mailboxes, struct __mailbox_node *node, void *msg);
 
 	void remove(MailboxList *mailboxes, struct __mailbox_node *node)
@@ -543,7 +552,7 @@ public:
 	}
 
 private:
-	bool send_max_locked(MailboxList *mailboxes, void *msg, size_t max,
+	bool send_max_locked(MailboxList *mailboxes, size_t max,
 						 struct list_head *task_list);
 	struct rb_root root_;
 	std::mutex mutex_;
@@ -613,8 +622,7 @@ WFMailboxTask *__NamedMailboxMap::create(const std::string& name,
 	return task;
 }
 
-bool __NamedMailboxMap::send_max_locked(MailboxList *mailboxes,
-										void *msg, size_t max,
+bool __NamedMailboxMap::send_max_locked(MailboxList *mailboxes, size_t max,
 										struct list_head *task_list)
 {
 	if (max == (size_t)-1)
@@ -635,7 +643,8 @@ bool __NamedMailboxMap::send_max_locked(MailboxList *mailboxes,
 	return true;
 }
 
-int __NamedMailboxMap::send(const std::string& name, void *msg, size_t max)
+int __NamedMailboxMap::send(const std::string& name, void *const msg[],
+							size_t max, int inc)
 {
 	LIST_HEAD(task_list);
 	struct __mailbox_node *node;
@@ -646,7 +655,7 @@ int __NamedMailboxMap::send(const std::string& name, void *msg, size_t max)
 	mutex_.lock();
 	mailboxes = __get_object_list<MailboxList>(name, &root_, false);
 	if (mailboxes)
-		erased = send_max_locked(mailboxes, msg, max, &task_list);
+		erased = send_max_locked(mailboxes, max, &task_list);
 
 	mutex_.unlock();
 	if (erased)
@@ -656,7 +665,8 @@ int __NamedMailboxMap::send(const std::string& name, void *msg, size_t max)
 	{
 		node = list_entry(task_list.next, struct __mailbox_node, list);
 		list_del(&node->list);
-		node->task->WFMailboxTask::send(msg);
+		node->task->WFMailboxTask::send(*msg);
+		msg += inc;
 		ret++;
 	}
 
@@ -691,9 +701,17 @@ WFMailboxTask *WFTaskFactory::create_mailbox_task(const std::string& name,
 	return __mailbox_map.create(name, std::move(callback));
 }
 
-int WFTaskFactory::send_by_name(const std::string& name, void *msg, size_t max)
+int WFTaskFactory::send_by_name(const std::string& name, void *msg,
+								size_t max)
 {
-	return __mailbox_map.send(name, msg, max);
+	return __mailbox_map.send(name, &msg, max, 0);
+}
+
+template<>
+int WFTaskFactory::send_by_name(const std::string& name, void *const msg[],
+								size_t max)
+{
+	return __mailbox_map.send(name, msg, max, 1);
 }
 
 /****************** Named Conditional ******************/
@@ -716,7 +734,7 @@ public:
 						  void **msgbuf);
 	WFConditional *create(const std::string& name, SubTask *task);
 
-	int signal(const std::string& name, void *msg, size_t max);
+	int signal(const std::string& name, void *const msg[], size_t max, int inc);
 	void signal(ConditionalList *conds, struct __conditional_node *node,
 				void *msg);
 
@@ -732,7 +750,7 @@ public:
 	}
 
 private:
-	bool signal_max_locked(ConditionalList *conds, void *msg, size_t max,
+	bool signal_max_locked(ConditionalList *conds, size_t max,
 						   struct list_head *cond_list);
 	struct rb_root root_;
 	std::mutex mutex_;
@@ -802,7 +820,7 @@ WFConditional *__NamedConditionalMap::create(const std::string& name,
 }
 
 bool __NamedConditionalMap::signal_max_locked(ConditionalList *conds,
-											  void *msg, size_t max,
+											  size_t max,
 											  struct list_head *cond_list)
 {
 	if (max == (size_t)-1)
@@ -823,7 +841,8 @@ bool __NamedConditionalMap::signal_max_locked(ConditionalList *conds,
 	return true;
 }
 
-int __NamedConditionalMap::signal(const std::string& name, void *msg, size_t max)
+int __NamedConditionalMap::signal(const std::string& name, void *const msg[],
+								  size_t max, int inc)
 {
 	LIST_HEAD(cond_list);
 	struct __conditional_node *node;
@@ -834,7 +853,7 @@ int __NamedConditionalMap::signal(const std::string& name, void *msg, size_t max
 	mutex_.lock();
 	conds = __get_object_list<ConditionalList>(name, &root_, false);
 	if (conds)
-		erased = signal_max_locked(conds, msg, max, &cond_list);
+		erased = signal_max_locked(conds, max, &cond_list);
 
 	mutex_.unlock();
 	if (erased)
@@ -844,7 +863,8 @@ int __NamedConditionalMap::signal(const std::string& name, void *msg, size_t max
 	{
 		node = list_entry(cond_list.next, struct __conditional_node, list);
 		list_del(&node->list);
-		node->cond->WFConditional::signal(msg);
+		node->cond->WFConditional::signal(*msg);
+		msg += inc;
 		ret++;
 	}
 
@@ -879,9 +899,16 @@ WFConditional *WFTaskFactory::create_conditional(const std::string& name,
 }
 
 int WFTaskFactory::signal_by_name(const std::string& name, void *msg,
-								   size_t max)
+								  size_t max)
 {
-	return __conditional_map.signal(name, msg, max);
+	return __conditional_map.signal(name, &msg, max, 0);
+}
+
+template<>
+int WFTaskFactory::signal_by_name(const std::string& name, void *const msg[],
+								  size_t max)
+{
+	return __conditional_map.signal(name, msg, max, 1);
 }
 
 /****************** Named Guard ******************/
