@@ -16,21 +16,50 @@
   Authors: Wang Zhulei (wangzhulei@sogou-inc.com)
 */
 
-
 #include <errno.h>
 #include <assert.h>
+#include <algorithm>
 #include "KafkaDataTypes.h"
 
+#define MIN(x, y)	((x) <= (y) ? (x) : (y))
+
 namespace protocol
 {
 
-#define MIN(x, y)	((x) <= (y) ? (x) : (y))
-
-static int compare_member(const void *p1, const void *p2)
+std::string KafkaConfig::get_sasl_info() const
 {
-	kafka_member_t *member1 = (kafka_member_t *)p1;
-	kafka_member_t *member2 = (kafka_member_t *)p2;
-	return strcmp(member1->member_id, member2->member_id);
+	std::string info;
+
+	if (strcasecmp(this->ptr->mechanisms, "plain") == 0)
+	{
+		info += this->ptr->mechanisms;
+		info += "|";
+		info += this->ptr->username;
+		info += "|";
+		info += this->ptr->password;
+		info += "|";
+	}
+	else if (strncasecmp(this->ptr->mechanisms, "SCRAM", 5) == 0)
+	{
+		info += this->ptr->mechanisms;
+		info += "|";
+		info += this->ptr->username;
+		info += "|";
+		info += this->ptr->password;
+		info += "|";
+	}
+
+	return info;
+}
+
+static bool compare_member(const kafka_member_t *m1, const kafka_member_t *m2)
+{
+	return strcmp(m1->member_id, m2->member_id) < 0;
+}
+
+inline void KafkaMetaSubscriber::sort_by_member()
+{
+	std::sort(this->member_vec.begin(), this->member_vec.end(), compare_member);
 }
 
 static bool operator<(const KafkaMetaSubscriber& s1, const KafkaMetaSubscriber& s2)
@@ -114,7 +143,7 @@ int KafkaCgroup::kafka_roundrobin_assignor(kafka_member_t **members,
 	int next = -1;
 
 	std::sort(subscribers->begin(), subscribers->end());
-	qsort(members, member_elements, sizeof (kafka_member_t *), compare_member);
+	std::sort(members, members + member_elements, compare_member);
 
 	for (const auto& subscriber : *subscribers)
 	{
