@@ -16,14 +16,14 @@
   Authors: Wang Zhulei (wangzhulei@sogou-inc.com)
 */
 
+#include <arpa/inet.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+#include <ctype.h>
 #include <openssl/sha.h>
 #include <openssl/hmac.h>
 #include <openssl/evp.h>
-#include <arpa/inet.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <ctype.h>
 #include "kafka_parser.h"
 
 static kafka_api_version_t kafka_api_version_queryable[] = {
@@ -610,13 +610,13 @@ int kafka_parser_append_message(const void *buf, size_t *size,
 
 	if (s > parser->message_size - parser->cur_size)
 	{
-		memcpy(parser->msgbuf + parser->cur_size, buf,
+		memcpy((char *)parser->msgbuf + parser->cur_size, buf,
 			   parser->message_size - parser->cur_size);
 		parser->cur_size = parser->message_size;
 	}
 	else
 	{
-		memcpy(parser->msgbuf + parser->cur_size, buf, s);
+		memcpy((char *)parser->msgbuf + parser->cur_size, buf, s);
 		parser->cur_size += s;
 	}
 
